#include "Maiden_struct.h"
#include "Maiden_funct.h"
#include "Constants.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <math.h>
#ifdef __APPLE__
#include "malloc.h"
#else
#include <malloc.h>
#endif

/*************************************************/
/* calc_tair() calculates daily air temperatures */
/*************************************************/
int calc_tair(const site_struct *site, const param_struct *par,
	meteo_struct *meteo, phenol_struct *phenol)
{
	int ok=1;
	int i, jj, y, yyy;
	double dz;
	double tmax, tmin;

	/* calculate elevation difference in kilometers */
	dz = (site->site_elev - site->base_elev)/1000.0;

	/* apply lapse rate corrections to tmax and tmin */
	/* Since tmax lapse rate usually has a larger absolute value than tmin
	lapse rate, it is possible at high elevation sites for these corrections
	to result in tmin > tmax. Check for that occurrence and force
	tmin = corrected tmax - 0.5 deg C. */
	for (i=0 ; i<ndays ; i++)
	  {
	    /* lapse rate corrections */
	    meteo->s_tmax[i] = tmax = meteo->tmax[i] + (dz * site->tmax_lr);
	    meteo->s_tmin[i] = tmin = meteo->tmin[i] + (dz * site->tmin_lr);

	    if (par->CO2_flag == 0) meteo->s_CO2[i] = 375.; // cc CO2 cste
	    else // cc CO2 from inmet PAD 301008
	      { meteo->s_CO2[i] = meteo->CO2[i]; } //375.; } //

	    /* average temperature */
	    meteo->s_tavg[i] = (tmax + tmin)/2.0 ;
	    
	    /* daylight temperature */
	    meteo->s_tday[i] = ((tmax - meteo->s_tavg[i])*TDAYCOEF) + meteo->s_tavg[i];

          
        ///* daylight temperature transformation FG*/
        if (i == 0 || i == 1)
        {
            meteo->s_tday_transf[i] = meteo->s_tday[i];
        }
        else if (i == ndays-1)
        {
            meteo->s_tday_transf[i-1] = ((meteo->s_tday[i]-(meteo->s_tday[i]-meteo->s_tday[i-1]+meteo->s_tday_transf[i-2]))/par->s_tday_tau)+meteo->s_tday_transf[i-2];
            meteo->s_tday_transf[i] = meteo->s_tday[i];
        }
        else
        {
            meteo->s_tday_transf[i-1] = ((meteo->s_tday[i]-(meteo->s_tday[i]-meteo->s_tday[i-1]+meteo->s_tday_transf[i-2]))/par->s_tday_tau)+meteo->s_tday_transf[i-2];
        }

	    /* night temperature */
	    //meteo->s_tnight[i] = (meteo->s_tday[i] + tmin)/2.0;

        /// FG : sum of avg temperature of summer
        y = meteo->year[i];
        if (meteo->yday[i] == 225)
        {
            meteo->T_previous_Y[y-year0]=0.;
            for (jj=0; jj<=25 ; jj++)
            {
                meteo->T_previous_Y[y-year0] += meteo->s_tavg[i-jj];
            }
        }
        /// FG : sum of degree days
        if (meteo->yday[i] > par->vegphase12)
        {
            phenol->sumdd[i] = phenol->sumdd[i-1];
            if (meteo->s_tavg[i] > par->temp_thres)
            {
                phenol->sumdd[i] += meteo->s_tavg[i]-par->temp_thres; //FG added "-par->temp_thres"
            }
            else
            {
                phenol->sumdd[i] += 0; //FG added "-par->temp_thres"
            }
        }
        else
        {
            phenol->sumdd[i] = 0;
        }
        if ((phenol->sumdd[i-1] >= par->gdd && phenol->sumdd[i-2] < par->gdd && phenol->day23[y-year0] == 0) || (meteo->yday[i] > par->vegphase23 && meteo->yday[i-1] == par->vegphase23 && phenol->day23[y-year0] == 0))
        {
            phenol->day23[y-year0] = meteo->yday[i];
        }
	  }

	  spline(meteo->T_previous_Y, meteo->T_previous_Y_sp, par->yearn - year0 + 1, 10);

	  for (yyy=0; yyy<(par->yearn - year0 + 1); yyy++)
	  {
	      meteo->T_previous_Y[yyy]=(meteo->T_previous_Y[yyy]-meteo->T_previous_Y_sp[yyy]);
	  }

	  //T_Score (meteo->T_previous_Y, par->yearn - year0 + 1 );
      if(meteo->T_previous_Y[0] != 0.){
            T_Score (meteo->T_previous_Y, par->yearn - year0 + 1 );
          }
    
        //TEST//tmax=Mean (meteo->T_previous_Y, par->yearn - year0 + 1 );
        //TEST//tmax=Samp_Std_Dev ( meteo->T_previous_Y, par->yearn - year0 + 1 , Mean (meteo->T_previous_Y, par->yearn - year0 + 1 ) );

      //std::vector<double> doubleVec(v_int.begin(), v_int.end());
	  if (par->day23_flex==1)
      {
          for (yyy=0; yyy<(par->yearn - year0 + 1); yyy++)
          {
              phenol->day23_sp[yyy]=(phenol->day23[yyy]);
          }
      }
	  else
      {
          spline(phenol->day23, phenol->day23_sp, par->yearn - year0 + 1, par->day23_flex);
          for (yyy=0; yyy<(par->yearn - year0 + 1); yyy++)
          {
              phenol->day23_sp[yyy]=round(phenol->day23_sp[yyy]);
          }
      }
	  ///


	return (!ok);
}
/* end of calc_tair() */

/****************************************************/
/* calc_prcp() calculates daily total precipitation */
/****************************************************/
int calc_prcp(const site_struct *site, const param_struct *par,
	meteo_struct *meteo)
{
	/* ------------------------ code history ------------------------------
     Precipitation
     ----------------------------------------------------------------------
     #---------------------------------------------------------------------
     #-- Partition of precipitation into liquid water and snow (by Ignacio)
     #-- I changed all the operations that used s_prcp (and that I did not 
	     intend to modify) to be s_prcp + s_prcp_snow, which amounts to the
	     same as it did before.
     #---------------------------------------------------------------------
   */
	int ok=1;
	int i, jj, y, yyy;
	double ratio, c, T_rain, T_snow;
	int CHOICE = 2;
    printf("prcp = %lf\n",meteo->prcp[0]);
	
	T_rain = 4.;
	T_snow = -2.;
    ratio = site->site_isoh / site->base_isoh;
    for (i=0 ; i<ndays ; i++){
      if(meteo->prcp[i] < 0.){
      	printf("day %d\t negative rain = %.24lf\n",i,meteo->prcp[i]);
      	system("pause");
	  }
    }
	
	if (ok)
	  {
		for (i=0 ; i<ndays ; i++)
		  { // divide precipitation in liquid and snow
			// meteo->s_prcp[i]      = meteo->prcp[i] * ratio; //version used before snow was implemented
			switch (CHOICE)
			  {
				case 1: //Simple model: precipitation above 1C is rainfall, otherwise it is snowfall
					c = ( meteo->s_tmax[i] - 1. ) / ( meteo->s_tmax[i] - meteo->s_tmin[i] ); // portion of day above freezing temperature
					if ( c >= 1. ) 
					  {
						meteo->s_prcp[i]      = meteo->prcp[i] * ratio;
						meteo->s_prcp_snow[i] = 0.;
					  }
					else if ( c <= 0. ) 
					  {
						meteo->s_prcp[i]      = 0.;
						meteo->s_prcp_snow[i] = meteo->prcp[i] * ratio;
					  }
					else 
					  {
						meteo->s_prcp[i]      = meteo->prcp[i] * ratio * c;
						meteo->s_prcp_snow[i] = meteo->prcp[i] * ratio * (1-c);
					  }
				break;
				
				case 2: //McCabe and Wolock (2009)
					c = ( T_rain - meteo->s_tavg[i] ) / ( T_rain - T_snow ); 
					if ( meteo->s_tavg[i] >= T_rain ) 
					  {
						meteo->s_prcp[i]      = meteo->prcp[i] * ratio;
						meteo->s_prcp_snow[i] = 0.;
					  }
					else if ( meteo->s_tavg[i] <= T_snow ) 
					  {
						meteo->s_prcp[i]      = 0.;
						meteo->s_prcp_snow[i] = meteo->prcp[i] * ratio;
					  }
					else 
					  {
						meteo->s_prcp[i]      = meteo->prcp[i] * ratio * (1-c);
						meteo->s_prcp_snow[i] = meteo->prcp[i] * ratio * c;
					  }
				break;
				
				case 3: //Leavesley et al (1996) //it obviously fails if tmax<Train and tmin<Tsnow, I do not know what they were thinking
					c = ( meteo->s_tmax[i] - T_snow ) / ( meteo->s_tmax[i] - meteo->s_tmin[i] ); 
					if ( meteo->s_tmax[i] >= T_rain ) 
					  {
						meteo->s_prcp[i]      = meteo->prcp[i] * ratio;
						meteo->s_prcp_snow[i] = 0.;
					  }
					else if ( meteo->s_tmax[i] <= T_snow ) 
					  {
						meteo->s_prcp[i]      = 0.;
						meteo->s_prcp_snow[i] = meteo->prcp[i] * ratio;
					  }
					else 
					  {
						meteo->s_prcp[i]      = meteo->prcp[i] * ratio * c;
						meteo->s_prcp_snow[i] = meteo->prcp[i] * ratio * (1-c);
					  }
				break;
			  }
            meteo->s_prcp_snow[i] = (1.-par->snowblow) * meteo->s_prcp_snow[i];
              
			/// FG : sum of avg precipitation of spring
			y = meteo->year[i];
			if (meteo->yday[i] == 110)
			  {
                meteo->Pspring[y-year0]=0.;
                for (jj=0; jj<=20 ; jj++)
				  {
					//meteo->Pspring[y-year0] += meteo->s_prcp[i-jj];
					meteo->Pspring[y-year0] += meteo->s_prcp[i-jj] + meteo->s_prcp_snow[i-jj];
				  }
			  }
		  }

        spline(meteo->Pspring, meteo->Pspring_sp, par->yearn - year0 + 1, 10);

        for (yyy=0; yyy<(par->yearn - year0 + 1); yyy++)
        {
          meteo->Pspring[yyy]=(meteo->Pspring[yyy]-meteo->Pspring_sp[yyy]);
        }
        if(meteo->Pspring[0] != 0.){
        T_Score (meteo->Pspring, par->yearn - year0 + 1 );
        }
            //TEST//tratio=Mean (meteo->Pspring, par->yearn - year0 + 1 );
            //TEST//tratio=Samp_Std_Dev ( meteo->Pspring, par->yearn - year0 + 1 , Mean (meteo->Pspring, par->yearn - year0 + 1 ) );
        ///
	  }
	return (!ok);
}
/* end of calc_prcp() */
/****************************************************/

/* calc_d18O2H_PV() calculates isotopes in daily precipitation and vapor PAD 280509 */
/****************************************************/
int calc_d18O2H_PV(const site_struct *site, const param_struct *par, const meteo_struct *meteo, d18O_struct *d18O, d2H_struct *d2H)
{
    int ok=1;
    int i;
    double tmean;
    printf("meteo_site : %i par->d18O_flag : %i (calc_d18O2H_PV in meteo.cpp)\n",par->meteo_site,par->d18O_flag);

    int CHOICE = 3;    // 1=Obs ; 2=REMOiso; 3=linear regression

    if (par->d18O_flag == 1 and par->meteo_site != 3) // Allow to compute d18O=f(T,P) pour Obs
    //if (par->d18O_flag == 1 and par->meteo_site == 3) // Allow to impose d18O=f(T,P) even for REMOrediso
    {
        printf("\n**** Daily d18OP and d18OV = fct(T,P) **** (meteo.cpp)\n\n");
        for (i=0 ; i<ndays ; i++)
        {
            tmean=(meteo->tmax[i] + meteo->tmin[i] ) /2.;

            if (meteo->prcp[i] > 0.)
            {
                switch (CHOICE)
                {
                    case 1:// Saclay PAD 280509
                        // 'SaclayDaily'
                        d18O->precip[i]=0.288993*tmean-0.242905*(meteo->s_prcp[i]+meteo->s_prcp_snow[i])*10.-13.800610; // (r=0.53)
                        break;

                    case 2:
                        d18O->precip[i]=0.348*tmean-0.292*(meteo->s_prcp[i]+meteo->s_prcp_snow[i])*10.-7.689;
                        if (i == 0) printf("**** d18OP = f(T,P) from **** REMOrediso-NOT-Adjusted ****\n");
                        break;

                    case 3:
                        // as in Lavergne et al. (2017)
                        //d18O->precip[i] = par->a_rain*tmean + par->b_rain*(meteo->s_prcp[i]+meteo->s_prcp_snow[i])*10. + par->c_rain; //b in mm^-1
                        
                        //d18O->snowfall[i] = par->a_snow*tmean + par->b_snow*(meteo->s_prcp[i]+meteo->s_prcp_snow[i])*10. + par->c_snow; //b in mm^-1
                        
                        d18O->precip[i] = par->a_rain*tmean + par->b_rain*(meteo->s_prcp[i]+meteo->s_prcp_snow[i]) + par->c_rain; //b in cm^-1
                        
                        d18O->snowfall[i] = par->a_snow*tmean + par->b_snow*(meteo->s_prcp[i]+meteo->s_prcp_snow[i]) + par->c_snow; //b in cm^-1
                        break;
                }
                // Saclay.xls PAD 280509
                d2H->precip[i]=6.8959*d18O->precip[i]+0.6995;
                d2H->snowfall[i]=6.8959*d18O->snowfall[i]+0.6995;
                // Use d18OV in equilibrium with d18OP
                d18O->vapor[i]=d18O->precip[i]-Eps_eq_d18O(tmean+273.15) ;
                d2H->vapor[i]=d2H->precip[i]-Eps_eq_d2H(tmean+273.15) ;
            }
            else //if (meteo->prcp[i] == 0.)
            {
                d18O->precip[i] = -9999.;
                d2H->precip[i] = -9999.;
                d18O->snowfall[i] = -9999.;
                d2H->snowfall[i] = -9999.;
                
                switch (CHOICE)
                {
                    case 1:
                        // Obs  'SaclayDaily' and Saclay.xls PAD 280509
                        d18O->vapor[i]=(0.280*tmean-9.24)-Eps_eq_d18O(tmean+273.15) ;
                        d2H->vapor[i]=(6.8959*(0.280*tmean-9.24)+.6995)-Eps_eq_d2H(tmean+273.15);
                        break;

                    case 2:
                        //00_aMultiReg.py ('SaclayREMOday')
                        d18O->vapor[i]=(0.336*tmean-8.388)-Eps_eq_d18O(tmean+273.15) ;
                        d2H->vapor[i]=(6.8959*(0.336*tmean-8.388)+.6995)-Eps_eq_d2H(tmean+273.15);
                        if (i<30) printf("**** d18OV=fct(d18OP) from ** REMOrediso-NOT-Adjusted ****\n");// PAD 190809
                        break;

                    case 3:
                        d18O->vapor[i]=(par->a_rain*tmean+par->c_rain)-Eps_eq_d18O(tmean+273.15) ;
                        d2H->vapor[i]=(6.8959*(par->a_rain*tmean+par->c_rain)+.6995)-Eps_eq_d2H(tmean+273.15);
                        break;
                }
            }
        }
    }
    else
    {// printf("\n**** Daily d18OP and d18OV from REMOiso ****\n"); }
        for (i=0 ; i<ndays ; i++)
        {
            if (meteo->prcp[i] > 0. && d18O->precip[i]<-1000.){
                printf("d18O->precip[%d]=%f\n",i,d18O->precip[i]);
            }
            d18O->snowfall[i]=d18O->precip[i];
            d2H->vapor[i]=d2H->precip[i]-Eps_eq_d2H(tmean+273.15) ;
            if (meteo->prcp[i] > 0)
            {
                d2H->precip[i]=6.8959*d18O->precip[i]+0.6995;
                d2H->snowfall[i]=6.8959*d18O->snowfall[i]+0.6995;
            }
            else
            {
                d2H->precip[i]=-9999.;
                d2H->snowfall[i]=-9999.;
            }
        }
    }
    
    return (!ok);
}
/* end of calc_d18O2H_PV() */
/**********************************************************************/
/* snowpack() estimates the accumulation and melt of snow for radiation
algorithm corrections */ 
///FG I'm not sure of the goodness of the units (newsnow = meteo->s_prcp[i]). meteo->s_prcp[i] should still be in cm/day here because it is converted in calc_srad_humidity_iterative. Need to check this.
/**********************************************************************/
int snowpack(const site_struct *site, const param_struct *par,
	meteo_struct *meteo)
{
	int ok=1;
	int i,count;
	int start_yday,prev_yday;
	double snowpack,newsnow,snowmelt,sum;
	
	/*Rv = R / MW; //from (m3 Pa mol-1 K-1) to [J.Kg-1 h2o K-1]
	pa = atm_pres(site->site_elev);
	t = meteo->s_tday[i];
	tk = t+273.15;
	t1 = t+0.2;
    t2 = t-0.2;
	pvs1 = 610.7 * exp(17.38 * t1 / (239.0 + t1));
	pvs2 = 610.7 * exp(17.38 * t2 / (239.0 + t2));
	s = (pvs1-pvs2) / (t1-t2);
	air_d = pa/(tk*Rv); // Pa/(t Kelvin * J Kg-1 h2o K-1)
	lhvap = 2.5023e6 - 2430.54 * t;
	gamma = CP * pa / (lhvap * EPS);
	ra = 400.;
	Epot_snow = (( air_d * CP * meteo->s_hum[i] / ra ) ) / ( s + gamma );*/

	/* first pass to initialize SWE array */
	snowpack = 0.0;
	for (i=0 ; i<ndays ; i++)
	{
		newsnow = 0.0;
		snowmelt = 0.0;
		//if (meteo->s_tmin[i] <= SNOW_TCRIT) newsnow = meteo->s_prcp[i] + meteo->s_prcp_snow[i];
		//else snowmelt = SNOW_TRATE * (meteo->s_tmin[i] - SNOW_TCRIT);
		newsnow = meteo->s_prcp_snow[i];
		if (meteo->s_tavg[i] > 0.) snowmelt = SNOW_TRATE * (meteo->s_tavg[i] - 0.); // cm/day
		snowpack += newsnow - snowmelt;
		if (snowpack < 0.0) snowpack = 0.0;
		meteo->s_swe[i] = snowpack;
	}

	/* use the first pass to set the initial snowpack conditions for the
	first day of data */
	start_yday = meteo->yday[0];
	if (start_yday == 1) prev_yday = 365;
	else prev_yday = start_yday-1;
	count = 0;
	sum = 0.0;
	for (i=1 ; i<ndays ; i++)
	  {
	    if (meteo->yday[i] == start_yday || meteo->yday[i] == prev_yday)
	      {
		count ++;
		sum += meteo->s_swe[i];
	      }
	  }
	/* Proceed with correction if there are valid days to reinitialize
	the snowpack estimates. Otherwise use the first-pass estimate. */
	if (count)
	  {
	    snowpack = sum/(double)count;
	    for (i=0 ; i<ndays ; i++)
	      {
		newsnow = 0.0;
		snowmelt = 0.0;
		//if (meteo->s_tmin[i] <= SNOW_TCRIT) newsnow = meteo->s_prcp[i] + meteo->s_prcp_snow[i];
		//else snowmelt = SNOW_TRATE * (meteo->s_tmin[i] - SNOW_TCRIT);
		newsnow = meteo->s_prcp_snow[i];
		if (meteo->s_tavg[i] > 0.) snowmelt = (SNOW_TRATE+0.02)* (meteo->s_tavg[i] - 0.);
		snowpack += newsnow - snowmelt -0.02;
		if (snowpack < 0.0) snowpack = 0.0;
		meteo->s_swe[i] = snowpack;
	      }
	  }

	return (!ok);
}
/* end of snowpack() */

/*************************************************************************/
/* without Tdew input data, an iterative estimation of shortwave radiation
   and humidity is required */
/*************************************************************************/
int calc_srad_humidity_iterative(const site_struct *site, const param_struct *par, meteo_struct *meteo, radiation_struct *rad)
{
	int ok=1;
	int i,j;
	int start_yday,end_yday,isloop;
	int ami,yday;
	double ttmax0[366];
	double flat_potrad[366];
	double slope_potrad[366];
	double daylength[366];
	double *dtr, *sm_dtr;
	double *parray, *window, *t_fmax, *tdew;
	double *save_pet;
	double sum_prcp,ann_prcp,effann_prcp;
	double sum_pet,ann_pet;
	double tmax,tmin;
	double aaa;//tk, FG
	double us;
	double t1,t2;
	double pratio;
	double lat,coslat,sinlat,dt,h,dh;
	double cosslp,sinslp,cosasp,sinasp;
	double bsg1,bsg2,bsg3;
	double decl,cosdecl,sindecl,cosegeom,sinegeom,coshss,hss;
	double sc,dir_beam_topa;
	double sum_flat_potrad,sum_slope_potrad,sum_trans;
	double cosh,sinh;
	double cza,cbsa,coszeh,coszwh;
	double dir_flat_topa,am;
	double pva,t_tmax,b;
	double tmink,pet,ratio,ratio2,ratio3,tdewk;
	double pvs,vpd;
	double trans1,trans2;
	double t_final,pdif,pdir,srad1,srad2;
	//double f_par_srad;misson 112305
	double pa;
	double sky_prop;
	double avg_horizon, slope_excess;
	double horizon_scalar, slope_scalar;
	double gaer,gb,radiation,atms_vp;
    int my_mw = 0;
    double Parid_eff_ann = 0.0;
    switch (model_version)
    {
        case 1: // Mediterranean
                my_mw = 120;
                Parid_eff_ann = 3.5;
        break;

        case 2: // Picea Mariana
                my_mw = 90;
                Parid_eff_ann = 8.0; //cm?
        break;
    }

	/* optical airmass by degrees */
	double optam[21] = {2.90,3.05,3.21,3.39,3.69,3.82,4.07,4.37,4.72,5.12,5.60,
	6.18,6.88,7.77,8.90,10.39,12.44,15.36,19.79,26.96,30.00};

	/* local array memory allocation */
	/* allocate space for DTR and smoothed DTR arrays */
	if (!(dtr = (double*) malloc(ndays * sizeof(double))))
	  {
	    printf("Error allocating for DTR array\n");
	    ok=0;
	  }
	if (!(sm_dtr = (double*) malloc(ndays * sizeof(double))))
	  {
	    printf("Error allocating for smoothed DTR array\n");
	    ok=0;
	  }
	/* allocate space for effective annual precip array */
	if (!(parray = (double*) malloc(ndays * sizeof(double))))
	  {
	    printf("Error allocating for effective annual precip array\n");
	    ok=0;
	  }
	/* allocate space for the prcp totaling array */
	if (!(window = (double*) malloc((ndays+my_mw)*sizeof(double))))
	  {
	    printf("Error allocating for prcp totaling array\n");
	    ok = 0;
	  }
	/* allocate space for t_fmax */
	if (!(t_fmax = (double*) malloc(ndays * sizeof(double))))
	  {
	    printf("Error allocating for p_tt_max array\n");
	    ok=0;
	  }
	/* allocate space for Tdew array */
	if (!(tdew = (double*) malloc(ndays * sizeof(double))))
	  {
	    printf("Error allocating for Tdew array\n");
	    ok=0;
	  }
	/* allocate space for save_pet array */
	if (!(save_pet = (double*) malloc(ndays * sizeof(double))))
	  {
	    printf("Error allocating for save_pet array\n");
	    ok=0;
	  }

	/* calculate diurnal temperature range for transmittance calculations */
	for (i=0 ; i<ndays ; i++)
	  {
	    tmax = meteo->tmax[i];
	    tmin = meteo->tmin[i];
		if (tmax < tmin) tmax = tmin+0.01;  //ggi: small correction 0.01 ot make it similar to inmet()
	    dtr[i] = tmax-tmin;
	  }

	/* smooth dtr array: After Bristow and Campbell, 1984 */
	if (ndays >= 30) /* use 30-day antecedent smoothing window */
	  {
	    if (pulled_boxcar(dtr, sm_dtr, ndays, 30, 0))
	      {
		printf("Error in boxcar smoothing, calc_srad_humidity()\n");
		ok=0;
	      }
	  }
	else /* smoothing window width = ndays */
	  {
	    if (pulled_boxcar(dtr, sm_dtr, ndays, ndays, 0))
	      {
		printf("Error in boxcar smoothing, calc_srad_humidity()\n");
		ok=0;
	      }
	  }

	/* calculate the annual total precip for decision between
	simple and arid-corrected humidity algorithm */
	sum_prcp = 0.0;
	for (i=0 ; i<ndays ; i++)
	  {
	    sum_prcp += meteo->s_prcp[i] + meteo->s_prcp_snow[i];
	  }
	ann_prcp = (sum_prcp/(double)ndays) * 365.25;
	if (ann_prcp == 0.0) ann_prcp = 1.0;

	/* Generate the effective annual precip, based on a 3-month
	   moving-window. Requires some special case handling for the
	   beginning of the record and for short records. */
	/* check if there are at least 90 days in this input file, if not,
	   use a simple total scaled to effective annual precip */
	if (ndays < my_mw)
	  {
	    sum_prcp = 0.0;
	    for (i=0 ; i<ndays ; i++)
	      {
		sum_prcp += meteo->s_prcp[i] + meteo->s_prcp_snow[i];
	      }
	    effann_prcp = (sum_prcp/(double)ndays) * 365.25;
	    /* if the effective annual precip for this period
	       is less than 8 cm, set the effective annual precip to 8 cm
	       to reflect an arid condition, while avoiding possible
	       division-by-zero errors and very large ratios (PET/Pann) */
        if (effann_prcp < Parid_eff_ann) effann_prcp = Parid_eff_ann;
	    for (i=0 ; i<ndays ; i++)
	      {
		parray[i] = effann_prcp;
	      }
	  }
	else
	  {
	    /* Check if the yeardays at beginning and the end of this input file
	       match up. If so, use parts of the three months at the end
	       of the input file to generate effective annual precip for
	       the first 3-months. Otherwise, duplicate the first 90 days
	       of the record. */
	    start_yday = meteo->yday[0];
	    end_yday = meteo->yday[ndays-1];
	    if (start_yday != 1)
	      {
		isloop = (end_yday == start_yday-1) ? 1 : 0;
	      }
	    else
	      {
		isloop = (end_yday == 365 || end_yday == 366) ? 1 : 0;
	      }

	    /* fill the first 90 days of window */
	    for (i=0 ; i<my_mw ; i++)
	      {
		if (isloop) window[i] = meteo->s_prcp[ndays-my_mw+i] + meteo->s_prcp_snow[ndays-my_mw+i];
		else window[i] = meteo->s_prcp[i] + meteo->s_prcp_snow[i];
	      }
	    /* fill the rest of the window array */
	    for (i=0 ; i<ndays ; i++)
	      {
		window[i+my_mw] = meteo->s_prcp[i] + meteo->s_prcp_snow[i];
	      }

	    /* for each day, calculate the effective annual precip from
	       scaled 90-day total */
	    for (i=0 ; i<ndays ; i++)
	      {
		sum_prcp = 0.0;
		for (j=0 ; j<my_mw ; j++)
		  {
		    sum_prcp += window[i+j];
		  }
		sum_prcp = (sum_prcp/my_mw) * 365.25;
		/* if the effective annual precip for this 90-day period
		   is less than 8 cm, set the effective annual precip to 8 cm
		   to reflect an arid condition, while avoiding possible
		   division-by-zero errors and very large ratios (PET/Pann) */
		parray[i] = (sum_prcp < Parid_eff_ann) ? Parid_eff_ann : sum_prcp;
	      }
	  } /* end if ndays >= 90 */

	/*****************************************
	 *                                       *
	 * start of the main radiation algorithm *
	 *                                       *
	 *****************************************/

	/* before starting the iterative algorithm between humidity and
	radiation, calculate all the variables that don't depend on
	humidity so they only get done once. */

	/* STEP (1) calculate pressure ratio (site/reference) = f(elevation) */

	/* LR_STD (-K m-1) standard temperature lapse rate */
	/* TSTD (K) standard temp at 0.0 m elevation */
	/* G_STD (m s-2) standard gravitational accel */
	/* R (m3 Pa mol-1 K-1) gas law constant */
	/* MA (kg mol-1) molecular weight of air */
	t1 = 1.0 - (LR_STD * site->site_elev)/T_STD;
	t2 = G_STD / (LR_STD * (R/MA));
	pratio = pow(t1,t2);

	/* STEP (2) correct initial transmittance for elevation */
	/* TBASE (dim) max inst. trans., 0m, nadir, dry atm */
	trans1 = pow(TBASE,pratio);

	/* STEP (3) build 366-day array of ttmax0, potential rad, and daylength */
	/* precalculate the transcendentals */
	lat = site->site_lat;

	/* check for (+/-) 90 degrees latitude, throws off daylength calc */
	/* RADPERDEG radians per degree */
	lat *= RADPERDEG;
	if (lat > 1.5707) lat = 1.5707;
	if (lat < -1.5707) lat = -1.5707;
	coslat = cos(lat);
	sinlat = sin(lat);
	cosslp = cos(site->site_slp * RADPERDEG);
	sinslp = sin(site->site_slp * RADPERDEG);
	cosasp = cos(site->site_asp * RADPERDEG);
	sinasp = sin(site->site_asp * RADPERDEG);

	/* cosine of zenith angle for east and west horizons */
	coszeh = cos(1.570796 - (site->site_ehoriz * RADPERDEG));
	coszwh = cos(1.570796 - (site->site_whoriz * RADPERDEG));

	/* sub-daily time and angular increment information */
	/* SRADDT timestep for radiation routine (600 seconds, 10 min) */
	/* SECPERRAD seconds per radian of hour angle */
	dt = SRADDT;                /* set timestep */
	dh = dt / SECPERRAD;        /* calculate hour-angle step */

	/* begin loop through yeardays */
	for (i=0 ; i<365 ; i++)
	{
		/* calculate cos and sin of declination */
		/* MINDECL minimum declination (radians) */
		/* DASOFF julian day offset of winter solstice */
		/* RADPERDAY radians of Earth orbit per julian day */
		decl = MINDECL * cos(((double)i + DAYSOFF) * RADPERDAY);
		cosdecl = cos(decl);
		sindecl = sin(decl);

		/* do some precalculations for beam-slope geometry (bsg) */
		bsg1 = -sinslp * sinasp * cosdecl;
		bsg2 = (-cosasp * sinslp * sinlat + cosslp * coslat) * cosdecl;
		bsg3 = (cosasp * sinslp * coslat + cosslp * sinlat) * sindecl;

		/* calculate daylength as a function of lat and decl */
		cosegeom = coslat * cosdecl;
		sinegeom = sinlat * sindecl;
		coshss = -(sinegeom) / cosegeom;
		if (coshss < -1.0) coshss = -1.0;  /* 24-hr daylight */
		if (coshss > 1.0) coshss = 1.0;    /* 0-hr daylight */
		hss = acos(coshss);                /* hour angle at sunset (radians) */
		/* daylength (seconds) */
		daylength[i] = 2.0 * hss * SECPERRAD;

		/* solar constant as a function of yearday (W/m^2) */
		sc = 1368.0 + 45.5*sin((2.0*PI*(double)i/365.25) + 1.7);
		/* extraterrestrial radiation perpendicular to beam, total over
		the timestep (J) */
		dir_beam_topa = sc * dt;

		sum_trans = 0.0;
		sum_flat_potrad = 0.0;
		sum_slope_potrad = 0.0;

		/* begin sub-daily hour-angle loop, from -hss to hss */
		for (h=-hss ; h<hss ; h+=dh)
		{
			/* precalculate cos and sin of hour angle */
			cosh = cos(h);
			sinh = sin(h);

			/* calculate cosine of solar zenith angle */
			cza = cosegeom * cosh + sinegeom;

			/* calculate cosine of beam-slope angle */
			cbsa = sinh * bsg1 + cosh * bsg2 + bsg3;

			/* check if sun is above a flat horizon */
			if (cza > 0.0)
			{
				/* when sun is above the ideal (flat) horizon, do all the
				flat-surface calculations to determine daily total
				transmittance, and save flat-surface potential radiation
				for later calculations of diffuse radiation */

				/* potential radiation for this time period, flat surface,
				top of atmosphere */
				dir_flat_topa = dir_beam_topa * cza;

				/* determine optical air mass */
				am = 1.0/(cza + 0.0000001);
				if (am > 2.9)
				{
					ami = (int)(acos(cza)/RADPERDEG) - 69;
					if (ami < 0) ami = 0;
					if (ami > 20) ami = 20;
					am = optam[ami];
				}

				/* correct instantaneous transmittance for this optical
				air mass */
				trans2 = pow(trans1,am);

				/* instantaneous transmittance is weighted by potential
				radiation for flat surface at top of atmosphere to get
				daily total transmittance */
				sum_trans += trans2 * dir_flat_topa;

				/* keep track of total potential radiation on a flat
				surface for ideal horizons */
				sum_flat_potrad += dir_flat_topa;

				/* keep track of whether this time step contributes to
				component 1 (direct on slope) */
				if ((h<0.0 && cza>coszeh && cbsa>0.0) ||
					(h>=0.0 && cza>coszwh && cbsa>0.0))
				{
					/* sun between east and west horizons, and direct on
					slope. this period contributes to component 1 */
					sum_slope_potrad += dir_beam_topa * cbsa;
				}

			} /* end if sun above ideal horizon */

		} /* end of sub-daily hour-angle loop */

		/* calculate maximum daily total transmittance and daylight average
		flux density for a flat surface and the slope */
		if (daylength[i] && sum_flat_potrad) 
		{
			ttmax0[i] = sum_trans / sum_flat_potrad;
			flat_potrad[i] = sum_flat_potrad / daylength[i];
			slope_potrad[i] = sum_slope_potrad / daylength[i];
		}
		else
		{
			ttmax0[i] = 0.0;
			flat_potrad[i] = 0.0;
			slope_potrad[i] = 0.0;
		}
	} /* end of i=365 days loop */

	/* force yearday 366 = yearday 365 */
	ttmax0[365] = ttmax0[364];
	flat_potrad[365] = flat_potrad[364];
	slope_potrad[365] = slope_potrad[364];
	daylength[365] = daylength[364];

	/* STEP (4)  calculate the sky proportion for diffuse radiation */
	/* uses the product of spherical cap defined by average horizon angle
	and the great-circle truncation of a hemisphere. this factor does not
	vary by yearday. */
	avg_horizon = (site->site_ehoriz + site->site_whoriz)/2.0;
	horizon_scalar = 1.0 - sin(avg_horizon * RADPERDEG);
	if (site->site_slp > avg_horizon) slope_excess = site->site_slp - avg_horizon;
	else slope_excess = 0.0;
	if (2.0*avg_horizon > 180.0) slope_scalar = 0.0;
	else
	{
		slope_scalar = 1.0 - (slope_excess/(180.0 - 2.0*avg_horizon));
		if (slope_scalar < 0.0) slope_scalar = 0.0;
	}
	sky_prop = horizon_scalar * slope_scalar;

	/* b parameter, and t_fmax not varying with Tdew, so these can be
	calculated once, outside the iteration between radiation and humidity
	estimates. Requires storing t_fmax in an array. */
	for (i=0 ; i<ndays ; i++)
	{
		/* b parameter from 30-day average of DTR */
		b = B0 + B1 * exp(-B2 * sm_dtr[i]);

		/* proportion of daily maximum transmittance */
		t_fmax[i] = 1.0 - 0.9 * exp(-b * pow(dtr[i],C));

		/* correct for precipitation if this is a rain day */
		if (meteo->prcp[i]) t_fmax[i] *= RAIN_SCALAR; /* RAIN_SCALAR(dim) correction to trans. for rain day */

	}

	/* As a first approximation, calculate radiation assuming
	that Tdew = Tmin */
	for (i=0 ; i<ndays ; i++)
	{
		yday = meteo->yday[i]-1;
		tdew[i] = meteo->s_tmin[i];
		pva = 610.7 * exp(17.38 * tdew[i] / (239.0 + tdew[i]));
		t_tmax = ttmax0[yday] + ABASE * pva; /* ABASE (1/Pa) vapor pressure effect on transmittance */

		/* final daily total transmittance */
		t_final = t_tmax * t_fmax[i];

		/* estimate fraction of radiation that is diffuse, on an
		instantaneous basis, from relationship with daily total
		transmittance in Jones (Plants and Microclimate, 1992)
		Fig 2.8, p. 25, and Gates (Biophysical Ecology, 1980)
		Fig 6.14, p. 122. */
		pdif = -1.25*t_final + 1.25;
		if (pdif > 1.0) pdif = 1.0;
		if (pdif < 0.0) pdif = 0.0;

		/* estimate fraction of radiation that is direct, on an
		instantaneous basis */
		pdir = 1.0 - pdif;

		/* the daily total radiation is estimated as the sum of the
		following two components:
		1. The direct radiation arriving during the part of
		   the day when there is direct beam on the slope.
		2. The diffuse radiation arriving over the entire daylength
		   (when sun is above ideal horizon).
		*/

		/* component 1 */
		srad1 = slope_potrad[yday] * t_final * pdir;

		/* component 2 (diffuse) */
		/* includes the effect of surface albedo in raising the diffuse
		radiation for obstructed horizons */
		srad2 = flat_potrad[yday] * t_final * pdif *
			(sky_prop + DIF_ALB*(1.0-sky_prop));

		/* snow pack influence on radiation */
		if (meteo->s_swe[i] > 0.0)
		{
			/* snow correction in J/m2/day */
			sc = (1.32 + 0.096 * meteo->s_swe[i]) * 1e6;
			/* convert to W/m2 and check for zero daylength */
			if (daylength[yday] > 0.0) sc /= daylength[yday];
			else sc = 0.0;
			/* set a maximum correction of 100 W/m2 */
			if (sc > 100.0) sc = 100.0;
		}
		else sc = 0.0;

		/* save daily radiation and daylength */
		if (par->meteo_site == 1) // or par->meteo_site == 3) // complete
		  {
		    meteo->s_srad[i] = meteo->rad[i];
		  }
		else
		  {
		    radiation = (srad1 + srad2 + sc);
		    meteo->s_srad[i] = radiation;
		  }
		radiation = (srad1 + srad2 + sc);
		meteo->srad_mtclim[i] = radiation + (-0.3105 * radiation + 11.498);

		/* misson */
		/* calculate PAR from Ross(1976) */
		rad->f_par_srad[i] = (0.60 + 0.42 * (pdir / pdif)) / (1 + (pdir / pdif));
		meteo->s_par[i] = meteo->s_srad[i] * rad->f_par_srad[i];

		meteo->s_dayl[i] = daylength[yday];

		//sensitivity misson
		if (test == 2) meteo->s_dayl[i] = 58000;
		if (test == 2) meteo->s_par[i] = meteo->s_srad[i] * 0.9;

	//	printf("%lf\n",meteo->s_srad[i]);
	}

	/* estimate annual PET first, to decide which humidity algorithm
	should be used */
	/* estimate air pressure at site */
	pa = atm_pres(site->site_elev);

	sum_pet = 0.0;
	for (i=0 ; i<ndays ; i++)
	{
	  if (par->meteo_site == 1) // or par->meteo_site == 3)
		{
		  meteo->s_ws[i] = meteo->ws[i]; // Assign wind speed (m s-1)
		}
		else
		{
		  meteo->s_ws[i] = par->ws_cst;
		}

		meteo->s_pa[i] = pa;  // Assign atmospheric pressure (Pa)
		save_pet[i] = calc_pet(meteo->s_srad[i],meteo->s_tday[i],pa,meteo->s_dayl[i]);
		sum_pet += save_pet[i];
	}
	ann_pet = (sum_pet/(double)ndays) * 365.25;

	/* humidity algorithm decision:
	PET/prcp >= 2.5 -> arid correction
	PET/prcp <  2.5 -> use tdew-tmin, which is already finished */
	//printf("PET/PRCP = %.4lf\n",ann_pet/ann_prcp);
	
	if (ann_pet/ann_prcp >= 2.5)
	{
		//printf("Using arid-climate humidity algorithm\n");
		/* Estimate Tdew using the initial estimate of radiation for PET */
		for (i=0 ; i<ndays ; i++)
		{
			tmink = meteo->s_tmin[i] + 273.15;
			pet = save_pet[i];

			/* calculate ratio (PET/effann_prcp) and correct the dewpoint */
			ratio = pet/parray[i];
			ratio2 = ratio*ratio;
			ratio3 = ratio2*ratio;
			tdewk = tmink*(-0.127 + 1.121*(1.003 - 1.444*ratio + 12.312*ratio2
				- 32.766*ratio3) + 0.0006*(dtr[i]));
			tdew[i] = tdewk - 273.15;
		}

		/* Revise estimate of radiation using new Tdew */
		for (i=0 ; i<ndays ; i++)
		{
			yday = meteo->yday[i]-1;
			pva = 610.7 * exp(17.38 * tdew[i] / (239.0 + tdew[i]));
			t_tmax = ttmax0[yday] + ABASE * pva;

			/* final daily total transmittance */
			t_final = t_tmax * t_fmax[i];

			/* estimate fraction of radiation that is diffuse, on an
			instantaneous basis, from relationship with daily total
			transmittance in Jones (Plants and Microclimate, 1992)
			Fig 2.8, p. 25, and Gates (Biophysical Ecology, 1980)
			Fig 6.14, p. 122. */
			pdif = -1.25*t_final + 1.25;
			if (pdif > 1.0) pdif = 1.0;
			if (pdif < 0.0) pdif = 0.0;

			/* estimate fraction of radiation that is direct, on an
			instantaneous basis */
			pdir = 1.0 - pdif;

			/* the daily total radiation is estimated as the sum of the
			following two components:
			1. The direct radiation arriving during the part of
			   the day when there is direct beam on the slope.
			2. The diffuse radiation arriving over the entire daylength
			   (when sun is above ideal horizon).
			*/

			/* component 1 */
			srad1 = slope_potrad[yday] * t_final * pdir;

			/* component 2 (diffuse) */
			/* includes the effect of surface albedo in raising the diffuse
			radiation for obstructed horizons */
			srad2 = flat_potrad[yday] * t_final * pdif *
				(sky_prop + DIF_ALB*(1.0-sky_prop));

			/* snow pack influence on radiation */
			if (meteo->s_swe[i] > 0.0)
			{
				/* snow correction in J/m2/day */
				sc = (1.32 + 0.096 * meteo->s_swe[i]) * 1e6;
				/* convert to W/m2 and check for zero daylength */
				if (daylength[yday] > 0.0) sc /= daylength[yday];
				else sc = 0.0;
			/* set a maximum correction of 100 W/m2 */
				if (sc > 100.0) sc = 100.0;
			}
			else sc = 0.0;

			if (par->meteo_site == 1) //or par->meteo_site == 3)
			  {
			    meteo->s_srad[i] = meteo->rad[i];
			  }
			else
			  {
			    radiation = (srad1 + srad2 + sc);
			    meteo->s_srad[i] = radiation;
			  }
			radiation = (srad1 + srad2 + sc);
			meteo->srad_mtclim[i] = radiation + (-0.3105 * radiation + 11.498);

			/* calculate PAR from Ross(1976) */
			rad->f_par_srad[i] = (0.60 + 0.42 * (pdir / pdif)) / (1 + (pdir / pdif));
			meteo->s_par[i] = meteo->s_srad[i] * rad->f_par_srad[i];

			if (test == 2) meteo->s_par[i] = meteo->s_srad[i] * 0.9;
		}

		/* Revise estimate of Tdew using new radiation */
		for (i=0 ; i<ndays ; i++)
		{
			tmink = meteo->s_tmin[i] + 273.15;
			pet = calc_pet(meteo->s_srad[i],meteo->s_tday[i],pa,meteo->s_dayl[i]);

			/* calculate ratio (PET/effann_prcp) and correct the dewpoint */
			ratio = pet/parray[i];
			ratio2 = ratio*ratio;
			ratio3 = ratio2*ratio;
			tdewk = tmink*(-0.127 + 1.121*(1.003 - 1.444*ratio + 12.312*ratio2
				- 32.766*ratio3) + 0.0006*(dtr[i]));
			tdew[i] = tdewk - 273.15;
		}
	} /* end of arid-correction humidity and radiation estimation */
	else
	  {
	    //gprintf("Using Tdew=Tmin humidity algorithm\n");
	  }

	/* now calculate vapor pressure from tdew */
	for (i=0 ; i<ndays ; i++)
	{
		/* output humidity as vapor pressure deficit (Pa) */
		/* calculate saturated VP at tday */
		//pvs = 610.7 * exp(17.38 * meteo->s_tday[i]/(239.0+meteo->s_tday[i]));
        pvs = 610.8 * exp(17.27 * meteo->s_tday[i]/(237.3+meteo->s_tday[i]));  // Allen et al. (1998)  AL 13032020
		meteo->s_svp[i] = pvs;


		if (par->meteo_site == 1) // or par->meteo_site == 3) // Obs or REMOcpl/CERFACScpl
		  {
		    // PAD 130809 Conversion de RH_avg en RH_daylight
		    printf("Verifier calcul RH dans meteorology.cpp 835 \n");
		    printf("      Pvap(Tavg) =?= PvapSat(Tdew) \n");
		    meteo->rh[i] = meteo->rhavg[i]*
		      610.7*exp(17.38*meteo->s_tavg[i]/(239.0+meteo->s_tavg[i]))
		      / pvs ;
		    pva = pvs * meteo->rh[i];
		    if (pva < 0.0) pva = 0.0;

		    printf("Pvap(Tavg): %lf =?= PvapSat(Tdew): %lf \n",meteo->rhavg[i] * (610.7 * exp(17.38 * meteo->s_tavg[i]/(239.0+meteo->s_tavg[i]))), 610.7 * exp(17.38 * tdew[i] / (239.0 + tdew[i])));

		  }
		else
		  {
		    atms_vp = 610.7 * exp(17.38 * tdew[i] / (239.0 + tdew[i]));
		    pva = atms_vp;
		    /* Estimation humidite relative moy sur 24h avec (PAD 050809)*/
		    /* http://fr.wikipedia.org/wiki/Point_de_rosee */
		    aaa = ( (tdew[i]+112.-(0.1*meteo->s_tavg[i])) / (112.+0.9*meteo->s_tavg[i])) ;
		    meteo->rhavg[i]= aaa*aaa*aaa*aaa*aaa*aaa*aaa*aaa ;

		    //printf("RHavg: %lf \n",meteo->rhavg[i] - pva / (610.7 * exp(17.38 * meteo->s_tavg[i]/(239.0+meteo->s_tavg[i]))));
		  }

		meteo->rh[i] = pva / pvs;

		atms_vp = 610.7 * exp(17.38 * tdew[i] / (239.0 + tdew[i]));
		meteo->avp_mtclim[i] = atms_vp + (0.0747 * atms_vp + 73.534);

		meteo->s_avp[i] = pva;  //Pa
		//vpd = pvs - pva;
        vpd = pvs - pva * pa/P_STD;  // corrected for elevation AL 13032020
		if (vpd < 0.0) vpd = 0.0;
		if (vpd > 4000.0) vpd = 4000.;

		if (test == 2)
		{
			meteo->s_hum[i] = 500.;
		}
		else
		{
			meteo->s_hum[i] = vpd;
		}

	} /* end for i = ndays loop */

	for (i=0 ; i<ndays ; i++)
	  {
	    /* leaf temperature in ?K */
	    //tk = meteo->s_tday[i] + 273.15; FG

	    // misson
	    // Boundary layer conductance from Montheit
	    // constant because wind speed is constant
	    us = 0.40*meteo->s_ws[i]/(log((zref-disd)/z0));
	    meteo->s_gbh[i] = gb = 1. / (6.2/pow(us,0.67));//m s-1
	    meteo->s_gbh[i] = gb * 1000./ MW;// mol m-2 s-1 of H2O
	    meteo->s_gbc[i] = meteo->s_gbh[i] / 1.37;// mol m-2 s-1 of CO2

	    // Canopy aerodynamic conductance from Montheit
	    // , constant because wind speed is constant
	    gaer = 1. / (meteo->s_ws[i]/(us*us)); //m s-1
	    meteo->s_gaer[i] = gaer * 1000. / MW; // (1000 = Kg m-3) mol m-2 s-1 of H2O

	    //printf("%lf %lf\n",gb,gaer);
	    //CO2 concentration: 360 Cyrille
	    // misson
	    // meteo->s_co2[i] = 375.;//(ppmv) moved in previous loop PAD 291008
	    meteo->s_prcp[i] = meteo->s_prcp[i] * 10.;//from cm to mm of H2O
	    meteo->s_prcp_snow[i] = meteo->s_prcp_snow[i] * 10.;//from cm to mm of snow
		//this is confusing, prcp should be transformed to mm right after reading, and the previous calculations corrected for this
	  }

	/* smooth day average temperature array as an estimation of soil temperature */
	if (ndays >= 30) /* use 30-day antecedent smoothing window */
	{
	  if (pulled_boxcar(meteo->s_tavg, meteo->s_surft, ndays, 30, 0))
	    {
	      printf("Error in boxcar smoothing, soil temperature()\n");
	      ok=0;
	    }
	}
	else /* smoothing window width = ndays */
	{
		if (pulled_boxcar(meteo->s_tavg, meteo->s_surft, ndays, ndays, 0))
		{
			printf("Error in boxcar smoothing, soil_temperature()\n");
			ok=0;
		}
	}

	/* free local array memory */
	free(dtr);
	free(sm_dtr);
	free(parray);
	free(window);
	free(t_fmax);
	free(tdew);
	free(save_pet);

	return (!ok);
}
/* end of calc_srad_humidity_iterative() */

/********************************************************************/
/* calc_pet() calculates the potential evapotranspiration for aridity
corrections in calc_vpd(), according to Kimball et al., 1997, used in
calc_srad_humidity_iterative */
/********************************************************************/
double calc_pet(double rad, double ta, double pa, double dayl)
{
	/* input parameters and units :
	double rad      (W/m2)  daylight average incident shortwave radiation
	double ta       (deg C) daylight average air temperature
	double pa       (Pa)    air pressure
	double dayl     (s)     daylength
	*/

	double rnet;       /* (W m-2) absorbed shortwave radiation avail. for ET */
	double lhvap;      /* (J kg-1) latent heat of vaporization of water */
	double gamma;      /* (Pa K-1) psychrometer parameter */
	double dt = 0.2;   /* offset for saturation vapor pressure calculation */
	double t1, t2;     /* (deg C) air temperatures */
	double pvs1, pvs2; /* (Pa)   saturated vapor pressures */
	double pet;        /* (kg m-2 day-1) potential evapotranspiration */
	double s;          /* (Pa K-1) slope of saturated vapor pressure curve */

	/* calculate absorbed radiation, assuming albedo = 0.2  and ground
	heat flux = 10% of absorbed radiation during daylight */
	rnet = rad * 0.72;

    /* calculate latent heat of vaporization as a function of ta */
    lhvap = 2.5023e6 - 2430.54 * ta;

    /* calculate the psychrometer parameter: gamma = (cp pa)/(lhvap epsilon)
    where:
    cp       (J/kg K)   specific heat of air
    epsilon  (unitless) ratio of molecular weights of water and air
    */
    gamma = CP * pa / (lhvap * EPS);

    /* estimate the slope of the saturation vapor pressure curve at ta */
    /* temperature offsets for slope estimate */
    t1 = ta+dt;
    t2 = ta-dt;

    /* calculate saturation vapor pressures at t1 and t2, using formula from
	Abbott, P.F., and R.C. Tabony, 1985. The estimation of humidity parameters.
	Meteorol. Mag., 114:49-56.
	*/
    pvs1 = 610.7 * exp(17.38 * t1 / (239.0 + t1));
    pvs2 = 610.7 * exp(17.38 * t2 / (239.0 + t2));

    /* calculate slope of pvs vs. T curve near ta */
    s = (pvs1-pvs2) / (t1-t2);

    /* calculate PET using Priestly-Taylor approximation, with coefficient
    set at 1.26. Units of result are kg/m^2/day, equivalent to mm water/day */
	pet = (1.26 * (s/(s+gamma)) * rnet * dayl)/lhvap;

	/* return a value in centimeters/day, because this value is used in a ratio
	to annual total precip, and precip units are centimeters */
	return (pet/10.0);
}
/* end of calc_pet */

/*****************************************************************************/
/* atm_pres() calculates the atmospheric pressure as a function of elevation,
used in calc_srad_humidity_iterative */
/*****************************************************************************/
double atm_pres(double elev)
{
	/* daily atmospheric pressure (Pa) as a function of elevation (m) */
	/* From the discussion on atmospheric statics in:
	Iribane, J.V., and W.L. Godson, 1981. Atmospheric Thermodynamics, 2nd
		Edition. D. Reidel Publishing Company, Dordrecht, The Netherlands.
		(p. 168)
	*/
	//int ok=1; FG
	double t1,t2;
	double pa;

	t1 = 1.0 - (LR_STD * elev)/T_STD;
	t2 = G_STD / (LR_STD * (R / MA));
	pa = P_STD * pow(t1,t2);

	return(pa);
}
/* end of atm_pres */

/***********************************************************************/
/* pulled_boxcar() calculates a moving average of antecedent values in an
array, using either a ramped (w_flag=1) or a flat (w_flag=0) weighting,
used in calc_srad_humidity_iterative  */
/***********************************************************************/
int pulled_boxcar(double *input,double *output,int n,int w,int w_flag)
{
	int ok=1;
    int i,j;
    double *wt = nullptr;
    double total,sum_wt;

    if (w > n) {
        printf("Boxcar longer than array...\n");
        printf("Resize boxcar and try again\n");
        ok=0;
    }

    if (ok && !(wt = (double*) malloc(w * sizeof(double))))
    {
    	printf("Allocation error in boxcar()\n");
    	ok=0;
    }

    if (ok)
    {
	    /* when w_flag != 0, use linear ramp to weight tails,
	    otherwise use constant weight */
	    sum_wt = 0.0;
	    if (w_flag)
	    {
	        for (i=0 ; i<w ; i++)
	       	{
	            wt[i] = (double)(i+1);
	            sum_wt += wt[i];
	        }
	    }
	    else
	    {
	        for (i=0 ; i<w ; i++)
	        {
	            wt[i] = 1.0;
	            sum_wt += wt[i];
	        }
	    }

	    /* fill the output array, starting with the point where a full
	    boxcar can be calculated */
	    for (i=w-1 ; i<n ; i++)
	    {
	        total = 0.0;
	        for (j=0 ; j<w ; j++)
	        {
	            total += input[i-w+j+1] * wt[j];
	        }
	        output[i] = total/sum_wt;
	    }

	    /* fill the first w elements of the output array with the value from
	    the first full boxcar */
	    for (i=0 ; i<w-1 ; i++)
	    {
	    	output[i] = output[w-1];
	    }

	    free(wt);

	} /* end if ok */

    return (!ok);
}
/* end of pulled_boxcar() */

